/*
 * Horizon Robotics
 *
 *  Copyright (C) 2020 Horizon Robotics Inc.
 *  All rights reserved.
 *  Author: leye.wang<leye.wang@horizon.ai>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __DIAG_LIB_INTERNAL_H
#define __DIAG_LIB_INTERNAL_H

#include <linux/netlink.h>
#include <sys/socket.h>
#include <errno.h>

#include "alivetick.h"
#include "diag_lib.h"
/* ============== Error code definition ============= */

/* ============== structure definition ============= */

#define  TESTLIB_MQ_NAME "/testlib_mq"
#define  SFMU_MQ_NAME "/sfmu_mq"
#define  SPI_MQ_NAME "/spi_mq"
#define  SERVICE_STARTUP_MQ_NAME "/service_startup_mq"
#define  SERVICE_WAKEUP_MQ_NAME "/service_wakeup_mq"

#define PIN_UP		1
#define PIN_DOWN	0
#define FUSA_FATAL_MAGIC 'f'
#define FUSA_SET_ERROR_PIN             _IOW(FUSA_FATAL_MAGIC, 0, unsigned int)

enum diag_err_status {
	DIAG_ERR_STATUS_UKNOWN = 1,
	DIAG_ERR_STATUS_OK,
	DIAG_ERR_STATUS_FAILED,
	DIAG_ERR_STATUS_MAX
};

/* ============== interfaces definition ============= */

/*
 * @Description: create netlink socket
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_netlink_init(void);

/*
 * @Description: destroy netlink socket
 * @sockfd: socket file descriptor
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_netlink_deinit(int32_t sockfd);

/*
 * @Description: receive message by netlink
 * @sockfd: socket file descriptor
 * @msg: message pointer
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_netlink_recv_msg(int32_t sockfd, diaglib_err_msg_t *msg);

/*
 * @Description: send message by netlink
 * @sockfd: socket file descriptor
 * @msg: message pointer
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_netlink_send_msg(int32_t sockfd, diaglib_err_msg_t *msg);

/*
 * @Description: create posix message queue
 * @mq_name: message queue name
 * @if_block: BLOCK or NON_BLOCK
 * @return: mq descriptor on success, system defined error on failure
 */
mqd_t diaglib_mq_create(const char *mq_name, uint32_t if_block);

/*
 * @Description: destroy posix message queue
 * @mq_name: message queue name to remove
 * @mq: message queue descriptor
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_mq_destroy(const char *mq_name, mqd_t mq);

/*
 * @Description: receive message by posix message queue
 * @mq: mq file descriptor
 * @msg: message pointer
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_mq_recv_msg(mqd_t mq, diaglib_err_msg_t *msg);

/*
 * @Description: send message by posix message queue
 * @mq: mq file descriptor
 * @msg: message pointer
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_mq_send_msg(mqd_t mq, diaglib_err_msg_t *msg);

/*
 * @Description: calculate the checksum of diaglib_err_t
 * @msg: message to be calculated
 * @return: sum of the message
 */
void diaglib_calculate_checksum(diaglib_err_msg_t *msg);

/*
 * @Description: open file
 * @pin: pin number
 * @return: fd on success, negative value on failure
 */
int32_t diaglib_operate_gpio_init(uint16_t pin);

/*
 * @Description: close file
 * @fd: file descriptor
 * @pin: pin number
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_operate_gpio_deinit(int32_t fd, uint16_t pin);

/*
 * @Description: operate gpio pin, pull up or down.
 * @fd: file descriptor
 * @opcode: pull up or pull down
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_operate_gpio_pin(int32_t fd, int8_t opcode);

/*
 * @Description: record diagnose information
 * @name: file name to store the information
 * @msg: message
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_record_diag_info(char *file_name, diaglib_err_msg_t *msg);

// /*
//  * @Description: verify program execute sequence
//  * @msg: alive tick message
//  * @return: 0 on success, negative value on failure
//  */
// int32_t diaglib_verify_program_seq(diaglib_alivetick_msg_t *msg);

/*
 * @Description: receive spi slience message from sfmu
 * msg: diagnose message
 * @return: 0 on success, others value on failure
 */
int32_t receive_spi_msg(diaglib_err_msg_t *msg);

/*
 * @Description: init error pin
 * @input:none
 * @return: fd on success, negative value on failure
 */
int32_t diaglib_error_pin_init();

/*
 * @Description: operate error pin, pull up or down.
 * @fd: file descriptor
 * @opcode: pull up or pull down
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_operate_error_pin(int32_t fd, uint32_t opcode);

/*
 * @Description: init error pin
 * @input:none
 * @return: 0 on success, negative value on failure
 */
int32_t diaglib_error_pin_deinit(int32_t fd);
int32_t hb_receive_service_startup_state(struct diag_service_state *state);

#define NETLINK_DIAG	23
#define NETLINK_PORT	100
#define NETLINK_GROUP	1

#define MAX_DATA_SIZE	150
#define ENV_PAYLOAD_SIZE 128

/*
 * The structures below shoule be same as corresponding structures in kernel
 * to parse received data from kernel.
 */

/*
 * event status.
 */

/* header part in struct diag_msg */
struct diag_msg_header {
	uint8_t msg_type;
	uint16_t msg_len;
	uint32_t checksum;
};

/* data part in struct diag_msg */
struct diag_msg_data {
	uint16_t module_id;
	uint16_t event_id;
	uint8_t event_sta;
	uint32_t sec;			// record occurrence_time of the event
	uint16_t msec;
	uint8_t event_level;
	uint16_t fchm_err_code;
	uint8_t payload[ENV_PAYLOAD_SIZE];
	uint8_t env_len;
};

/*
 * message received from kernel space
 */
struct diag_msg {
	struct diag_msg_header header;
	struct diag_msg_data data;
};

/*
 * message from netlink: nlmsghdr + data
 */
struct netlink_msg {
	struct nlmsghdr hdr;
	uint8_t data[MAX_DATA_SIZE];
};

#endif //__DIAG_LIB_INTERNAL_H
